﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2015 Joseph Helm and Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Problems.BinPacking.Dimensions;
using HeuristicLab.Problems.BinPacking.Interfaces;
using HeuristicLab.Problems.BinPacking.PackingBin;
using HeuristicLab.Problems.BinPacking.PackingItem;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Core;
using HeuristicLab.Common;
using HeuristicLab.Problems.BinPacking.Evaluators;
using HeuristicLab.Encodings.PermutationEncoding;
using HeuristicLab.Encodings.PackingEncoding.PackingSequence;
using HeuristicLab.Encodings.PackingEncoding.GroupingVector;
using HeuristicLab.Problems.Instances;
using HeuristicLab.Encodings.PackingEncoding.MultiComponentVector;
using HeuristicLab.PluginInfrastructure;

namespace HeuristicLab.Problems.BinPacking.Problem {
  [Item("Bin Packing Problem (3D, identical cuboids) (BPP)", "Represents a three-dimensional bin-packing problem using only bins with identical measures and bins/items with cuboidic shapes.")]
  [StorableClass]
  [Creatable(CreatableAttribute.Categories.CombinatorialProblems, Priority = 310)]
  public class CuboidIdenticalBinPackingProblem : RegularIdenticalBinPackingProblem<ThreeDimensionalPacking, CuboidPackingBin, CuboidPackingItem> {

    #region Default Instance
    private static readonly BPPData DefaultInstance = new BPPData() {  
      Name = "3D BPP Default Instance", 
      Description = "The default instance for 3D Bin Packing.",
      BinMeasures = new int[] {25,25,35},
      ItemMeasures = new int[][] {
        new int[] {12,5,10},
        new int[] {10,18,20},
        new int[] {9,7,7},
        new int[] {21,12,4},
        new int[] {8,8,12},
        new int[] {3,6,14},
        new int[] {20,4,9},
        new int[] {5,9,8},
        new int[] {7,17,3},
        new int[] {13,20,15},
        new int[] {9,11,9},
        new int[] {10,18,20},
        new int[] {9,7,7},
        new int[] {21,12,4},
        new int[] {8,8,12},
        new int[] {3,6,14},
        new int[] {20,4,9},
        new int[] {5,9,8},
        new int[] {7,17,3},
        new int[] {13,20,15},
        new int[] {9,11,9},
        new int[] {10,18,20},
        new int[] {9,7,7},
        new int[] {21,12,4},
        new int[] {8,8,12},
        new int[] {3,6,14},
        new int[] {20,4,9},
        new int[] {5,9,8},
        new int[] {7,17,3},
        new int[] {13,20,15},
        new int[] {9,11,9},
        new int[] {10,18,20},
        new int[] {9,7,7},
        new int[] {21,12,4},
        new int[] {8,8,12},
        new int[] {3,6,14},
        new int[] {20,4,9},
        new int[] {5,9,8},
        new int[] {7,17,3},
        new int[] {13,20,15},
        new int[] {9,11,9}
      },
      Items = 30
    };
    #endregion
    
    [StorableConstructor]
    protected CuboidIdenticalBinPackingProblem(bool deserializing) : base(deserializing) { }
    protected CuboidIdenticalBinPackingProblem(CuboidIdenticalBinPackingProblem original, Cloner cloner)
      : base(original, cloner) {
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new CuboidIdenticalBinPackingProblem(this, cloner);
    }
    public CuboidIdenticalBinPackingProblem() : base(
      new PackingPlanEvaluationAlgorithm<Permutation, ThreeDimensionalPacking, CuboidPackingBin, CuboidPackingItem>()) {
    }



    #region Helpers
    protected override void InitializeDecoder() {
      Operators.RemoveAll(op => op is I2DOperator);

      PackingSolutionDecoderParameter.ValidValues.Clear();
      if (SolutionCreator is PackingSequenceRandomCreator) {   
        PackingSolutionDecoderParameter.ValidValues.UnionWith(ApplicationManager.Manager.GetInstances<I3DPSDecoder>());
      } else if (SolutionCreator is GroupingVectorRandomCreator) {      
        PackingSolutionDecoderParameter.ValidValues.UnionWith(ApplicationManager.Manager.GetInstances<I3DGVDecoder>());
      } else if (SolutionCreator is MultiComponentVectorRandomCreator) {    
        PackingSolutionDecoderParameter.ValidValues.UnionWith(ApplicationManager.Manager.GetInstances<I3DMCVDecoder>());
      } else {
        string error = "The given problem does not support the selected solution-creator.";
        ErrorHandling.ShowErrorDialog(error, null);
      }
    }

    protected override IPackingPlanEvaluator CreateDefaultEvaluator() {
      return new PackingRatioCuboidIdenticalBinEvaluator();
    }

    protected override void InitializeProblemData() {
      Load(DefaultInstance);
    }

    protected override void RemoveTooBigItems() {
      PackingItemMeasures.RemoveAll(pi => 
        !PackingBinMeasures.Encloses (new ThreeDimensionalPacking(0, 0, 0, 0, false), pi) &&
        !PackingBinMeasures.Encloses(new ThreeDimensionalPacking(0, 0, 0, 0, true), pi));
    }
    #endregion
  }
}
